---
elm:
  dependencies:
    vicramgon/logicus : latest
---


# Problema del robo

> Fuente: Ejercios de Lógica Computacional
> (https://www.cs.us.es/~fsancho/?p=logica-informatica-2020-21)

Alberto, Berta y Carlos son los tres sospechosos de un robo. Se les interroga por separado y éstas son sus declaraciones:

- _Alberto_: Berta es culpable y Carlos es inocente

- _Berta_: Si Alberto es culpable, Carlos también.

- _Carlos_: Yo soy inocente, pero al menos uno de los otros dos es culpable

¿Son consistentes las declaraciones?¿quién es inocente y quién culpable si todos dijeron la verdad?¿y si únicamente lo hicieron las personas inocentes?

## Solución

Sean $a, b, c$ variables proposicionales que indican si Alberto, Berta y Carlos son culpables (V) o si son inocentes (F). De forma que los testimonios podrían expresarse como:

$$ \{b \wedge \neg c, \, a \rightarrow c, \, \neg c \wedge (a \vee b) \} $$

Ahora, determinar la consistencia del conjunto es equivalente a determinar la satisfactibilidad de la conjunción de las fórmulas. Para lo que utilizaremos la FND. Haciendo uso de LogicUS:

```elm {l id="imports"}
import LogicUS.PL.SintaxSemantics exposing (..)
import LogicUS.PL.Clauses exposing (..)

fplRead : String -> FormulaPL
fplRead = fplReadExtraction << fplReadFromString
```

Definimos las fórmulas:

```elm {l context="1" follows="imports"}
f1 : FormulaPL
f1 = fplRead "b & ¬ c"

f2 : FormulaPL
f2 = fplRead "a -> c"

f3 : FormulaPL
f3 = fplRead "¬c & (a | b)"

```

^^^elm{m=("$$ " ++ (String.join "," <| List.map fplToMathString [f1,f2,f3]) ++ " $$") context="1"}^^^

Entonces ahora para determinar la satisfactibilidad podemos hacerlo a través del uso de cláusulas por lo que transformaremos el conjunto en un conjunto de cláusulas proposicionales con el uso de la función `splToClauses`

```elm {l r context="1"}
cs : ClausePLSet
cs = splToClauses [f1,f2,f3]
```

<br> Que corresponde a una lista de cláusulas correspondientes a listas de literales expresados como pares _(símbolo,signo)_. Representándolas:

^^^elm{m=("$$ " ++ (csplToMathString cs) ++ " $$") context="1"}^^^

De forma que veamos si es consistente, por fuerza bruta, evaluando todas las posibles interpretaciones para el conjunto, con la función `csplModels`, de forma que si el conjunto de modelos es vacío, entonces será insatisfactible y no lo será en caso contrario.

```elm {l context="1"}
csModels : List Interpretation
csModels =  csplModels cs

```

^^^elm{m=("$$ " ++ (interpretationsToMathString csModels [("a",[]),("b",[]),("c",[])]) ++ " $$") context="1"}^^^

Por tanto la culpable, en caso de que todos los testimonios sean ciertos, sería Berta.

En el otro supuesto la operativa sería análoga pero con el conjunto de fórmulas:

$$ \{(b \wedge \neg c) \leftrightarrow \neg a, \, (a \rightarrow c) \leftrightarrow \neg b, \, (\neg c \wedge (a \vee b))\leftrightarrow \neg c \} $$


```elm {l context="2" follows="imports"}
f1 : FormulaPL
f1 = fplRead "(b & ¬ c) <-> ¬ a"

f2 : FormulaPL
f2 = fplRead "(a -> c)  <-> ¬ b"

f3 : FormulaPL
f3 = fplRead "(¬c & (a | b))  <-> ¬ c"

```

Que tiene asociado el conjunto de cláusulas 

```elm {l m=("$ " ++ (csplToMathString cs) ++ " $") context="2"}
cs : ClausePLSet
cs = splToClauses [f1,f2,f3]
```

Y éste, a su vez, el conjunto de modelos:

```elm {l m=("$$ " ++ (interpretationsToMathString csModels [("a",[]),("b",[]),("c",[])]) ++ " $$") context="2"}
csModels : List Interpretation
csModels =  csplModels cs
```

Por tanto los culpables habrían acusado falsamente a Berta.
