# LogicUS

## Design Goals

This package has been created to work with computer logic, specifically with propositional logic and first-order logic (in development). For a series of types and functions have been created that allow the definition and interpretation of formulas as well as the application of the main algorithms to reduce the consistency of formulas and sets.

## Overview

Modules designed provides the tools that allow the definition, execution and visualization of main algorithms in the regarding area. In concrete:


### Propositional Logic

The _LogicUS.PL_ packages allow the definition of formulas and propositional sets on which a series of also implemented solving algorithms can be applied. Next, we make a synopsis of each of the modules exposed:

- **_LogicUS.PL.SintaxSemantics_**: It constitutes the basis of propositional logic, so that it exposes the syntax for the definition of the formulas as well as the semantics for their interpretation as well as some basic algorithm in the field of propositional logic.

- **_LogicUS.PL.SemanticTableaux_**: It develops all the necessary tools for working with semantic boards in PL, distinguishing the different types of formulas and rules and also allowing the visualization of the complete board.

- **_LogicUS.PL.NormalForms_**: It contains the functions necessary for the transformation of formulas into normal forms (negative, conjunctive and disjunctive).

- **_LogicUS.PL.Clauses_**: It provides some functions that allow work with propositional clauses, definition, operations, transformation of formulas and sets into clausal sets, ...

- **_LogicUS.PL.DPLL_**: It defines the functions necessary for the application of the DPLL solving algorithm to sets of propositional clauses as well as the search for models, based on this technique. Also, it allows the visualization of the complete board.

- **_LogicUS.PL.CSP_**: It defines the functions necessary for working with BigFormulas including its definition and a basic SAT Solver (based in Backtracking + MRV) for solving some (little) CSP.

- **_LogicUS.PL.Resolution_**: Define the functions for working with the resolution algorithms implementing different strategies: saturation, regular, best first, linear, positive, negative, unitary, by inputs, ...

### First Order Logic

The _LogicUS.FOL_ packages allow working with formulas and sets in First Order Logic. It includes aspects like L-Structures valuation, substitution performing, normal forms, clauses and other algorithms like Semantic Tableaux, Herbrand Works or Resolution. Next, we make a synopsis of each of the modules exposed:

- **_LogicUS.FOL.SintaxSemantics_**:
  It constitutes the basis of first logic, so that it exposes the syntax for the definition of the formulas and substituions (including all the operations with it, like clausre, variable renaming, ...) as well as the semantics for the L-Structure and its evaluation.

- **_LogicUS.FOL.SemanticTableaux_**: It develops all the necessary tools for working with semantic boards in FOL, distinguishing the different types of formulas and rules and also allowing the visualization of the complete board.

- **_LogicUS.FOL.NormalForms_**: It contains the functions necessary for the transformation of formulas into normal forms: Prenex, Skolem, NNF, CNF, DNF.

- **_LogicUS.FOL.Clauses_**: It provides some functions that allow work with first order clauses, definition, operations, transformation of formulas and sets into clausal sets, ...

- **_LogicUS.FOL.Herbrand_**: It defines the functions necessary for the application of the Herbrand Works (Universe, interpretations, models, ...)

- **_LogicUS.FOL.Unification_**: Define the functions for working with the unification algorithms: terms and atoms MGU.

- **_LogicUS.FOL.Resolution_**: Define the functions for working with the resolution algorithm in FOL and its representation. (Future features: resolution strategies).
